"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const fastDeepEqual = require("fast-deep-equal");
const mobx_1 = require("mobx");
const utils_1 = require("./utils");
const validation_1 = require("./validation");
class Field {
    constructor(defaultValue, { validate, debounce = false, disabled = false, validationContext = validation_1.defaultValidationContext, formater, parser, label, }) {
        this._value = null;
        this._interacted = false;
        this.disabled = false;
        this._isValidFn = [];
        this.markAsTouched = () => {
            if (!this._interacted) {
                this._interacted = true;
            }
        };
        this.markAsUntouched = () => {
            if (this._interacted) {
                this._interacted = false;
            }
        };
        this.setValue = (val, { interract = true, isInitValue = false } = {}) => {
            if (interract && !this._interacted) {
                this._interacted = true;
            }
            this._value = val;
            if (isInitValue) {
                this.initValue = this.value;
            }
        };
        this.onInputChange = (evt, value) => {
            if (evt && typeof value === "undefined") {
                value = evt.target.value;
            }
            const parsedValue = this.parser ? this.parser(value) : value;
            if (parsedValue !== this._value) {
                this.setValue(parsedValue, { interract: false });
            }
        };
        this.validate = async () => {
            const validationContext = this.validationContext(this._value, this.label);
            let lastRes;
            for (let i = 0; i < this._isValidFn.length; i++) {
                const isValidFn = this._isValidFn[i];
                let res = isValidFn(validationContext);
                if (utils_1.isPromise(res)) {
                    res = await res;
                }
                if (!res || !(typeof res === "object")) {
                    throw new Error("Invalid validator result");
                }
                lastRes = res;
                if (!res.valid || res.stop) {
                    break;
                }
            }
            if (lastRes) {
                this.setValidation(lastRes);
            }
        };
        this.disabled = disabled;
        this.formater = formater;
        this.parser = parser;
        this.validationContext = validationContext;
        this.defaultValue = defaultValue;
        this.initValue = defaultValue;
        this.setValue(defaultValue, { interract: false });
        if (label) {
            this.label = label;
        }
        if (validate) {
            if (validate instanceof Array) {
                this._isValidFn = validate;
            }
            else {
                this._isValidFn = [validate];
            }
        }
        if (this._isValidFn.length) {
            this._validation = { valid: false };
            mobx_1.autorun(this.validate, { delay: debounce ? 300 : 0 });
        }
        else {
            this._validation = { valid: true };
        }
    }
    reset({ untouched = false, defaultValue = false } = {}) {
        if (defaultValue) {
            this.initValue = this.defaultValue;
        }
        this._value = this.initValue;
        if (untouched) {
            this.markAsUntouched();
        }
    }
    setDisabled(value) {
        this.disabled = value;
    }
    get value() {
        return this._value;
    }
    get inputValue() {
        return this.formater ? this.formater(this._value) : this._value;
    }
    setValidation(validation) {
        this._validation = validation;
    }
    get errorMessage() {
        if (!this._interacted || !this._validation) {
            return undefined;
        }
        return this._validation?.message;
    }
    get isValid() {
        return this._validation.valid;
    }
    get validation() {
        return this._validation;
    }
    get interacted() {
        return this._interacted;
    }
    get pristine() {
        return fastDeepEqual(this.initValue, this.value);
    }
    onChange(fn) {
        this._dispose = mobx_1.reaction(() => this.value, (value) => {
            fn(value);
        });
    }
    destroy() {
        if (this._dispose) {
            this._dispose();
        }
    }
}
__decorate([
    mobx_1.observable,
    __metadata("design:type", Object)
], Field.prototype, "_value", void 0);
__decorate([
    mobx_1.observable,
    __metadata("design:type", Object)
], Field.prototype, "_interacted", void 0);
__decorate([
    mobx_1.observable,
    __metadata("design:type", Object)
], Field.prototype, "_validation", void 0);
__decorate([
    mobx_1.observable,
    __metadata("design:type", Object)
], Field.prototype, "disabled", void 0);
__decorate([
    mobx_1.observable,
    __metadata("design:type", Object)
], Field.prototype, "initValue", void 0);
__decorate([
    mobx_1.action("FIELD_RESET"),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], Field.prototype, "reset", null);
__decorate([
    mobx_1.action("FIELD_SET_DISABLED"),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Boolean]),
    __metadata("design:returntype", void 0)
], Field.prototype, "setDisabled", null);
__decorate([
    mobx_1.action("FIELD_MARK_AS_TOUCH"),
    __metadata("design:type", Object)
], Field.prototype, "markAsTouched", void 0);
__decorate([
    mobx_1.action("FIELD_MARK_AS_UNTOUCHED"),
    __metadata("design:type", Object)
], Field.prototype, "markAsUntouched", void 0);
__decorate([
    mobx_1.computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], Field.prototype, "value", null);
__decorate([
    mobx_1.action("SET_FIELD_VALUE"),
    __metadata("design:type", Object)
], Field.prototype, "setValue", void 0);
__decorate([
    mobx_1.computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], Field.prototype, "inputValue", null);
__decorate([
    mobx_1.action("SET_FIELD_VALIDATION"),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], Field.prototype, "setValidation", null);
__decorate([
    mobx_1.computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], Field.prototype, "errorMessage", null);
__decorate([
    mobx_1.computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], Field.prototype, "isValid", null);
__decorate([
    mobx_1.computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], Field.prototype, "pristine", null);
exports.default = Field;
