"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const mobx_1 = require("mobx");
const fastDeepEqual = require("fast-deep-equal");
class FieldsObject {
    constructor(fields, { validate, data } = {}) {
        this.validate = validate;
        this.fields = mobx_1.observable.object(fields, {}, { deep: false });
        if (data) {
            this.setValue(data, { interract: false });
        }
        this.defaultValue = this.value;
        this.initValue = this.value;
    }
    reset({ untouched = false, defaultValue = false } = {}) {
        if (defaultValue) {
            this.initValue = this.defaultValue;
        }
        this.setValue(this.initValue);
        if (untouched) {
            this.markAsUntouched();
        }
    }
    markAsTouched() {
        Object.values(this.fields).forEach((field) => field.markAsTouched());
    }
    markAsUntouched() {
        Object.values(this.fields).forEach((field) => field.markAsUntouched());
    }
    setValue(newData, { interract = false, isInitValue = false } = {}) {
        if (!newData) {
            return;
        }
        Object.keys(this.fields).forEach((fieldName) => {
            if (newData[fieldName] !== undefined) {
                this.fields[fieldName].setValue(newData[fieldName], {
                    interract,
                    isInitValue,
                });
            }
        });
        if (isInitValue) {
            this.initValue = this.value;
        }
    }
    get validation() {
        if (this.validate) {
            const validation = this.validate(this.value);
            if (!validation.valid) {
                return validation;
            }
        }
        const invalidField = Object.values(this.fields).find((field) => !field.isValid);
        if (invalidField) {
            return invalidField.validation;
        }
        return {
            valid: true,
        };
    }
    get isValid() {
        return this.validation.valid;
    }
    get value() {
        const res = {};
        Object.keys(this.fields).forEach((fieldName) => {
            res[fieldName] = this.fields[fieldName].value;
        });
        return res;
    }
    get interacted() {
        return Object.values(this.fields).some((field) => field.interacted);
    }
    get pristine() {
        return fastDeepEqual(this.initValue, this.value);
    }
    get errorMessage() {
        return this.validation?.message;
    }
}
__decorate([
    mobx_1.observable,
    __metadata("design:type", Object)
], FieldsObject.prototype, "fields", void 0);
__decorate([
    mobx_1.observable,
    __metadata("design:type", Object)
], FieldsObject.prototype, "initValue", void 0);
__decorate([
    mobx_1.action("FIELD_OBJECT_RESET"),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], FieldsObject.prototype, "reset", null);
__decorate([
    mobx_1.action("FIELD_OBJECT_MAT"),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], FieldsObject.prototype, "markAsTouched", null);
__decorate([
    mobx_1.action("FIELD_OBJECT_MAUT"),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], FieldsObject.prototype, "markAsUntouched", null);
__decorate([
    mobx_1.action("FIELD_OBJECT_SETVALUE"),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", void 0)
], FieldsObject.prototype, "setValue", null);
__decorate([
    mobx_1.computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], FieldsObject.prototype, "validation", null);
__decorate([
    mobx_1.computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], FieldsObject.prototype, "isValid", null);
__decorate([
    mobx_1.computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], FieldsObject.prototype, "value", null);
__decorate([
    mobx_1.computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], FieldsObject.prototype, "interacted", null);
__decorate([
    mobx_1.computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], FieldsObject.prototype, "pristine", null);
__decorate([
    mobx_1.computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], FieldsObject.prototype, "errorMessage", null);
exports.default = FieldsObject;
