"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const fastDeepEqual = require("fast-deep-equal");
const mobx_1 = require("mobx");
class FieldsArray {
    constructor(defaultValue, { validate, builder } = {}) {
        this.validate = validate;
        this.builder = builder;
        this.fields = mobx_1.observable.array(defaultValue, { deep: false });
        this.defaultValue = this.value;
        this.initValue = this.value;
    }
    reset({ untouched = false, defaultValue = false } = {}) {
        if (defaultValue) {
            this.initValue = this.defaultValue;
        }
        this.setValue(this.initValue);
        // this.setValue(this.initValue as FieldsArrayData<TField>[]);
        if (untouched) {
            this.markAsUntouched();
        }
    }
    markAsTouched() {
        this.fields.forEach((field) => field.markAsTouched());
    }
    markAsUntouched() {
        this.fields.forEach((field) => field.markAsUntouched());
    }
    setValue(data, { interract = false, isInitValue = false } = {}) {
        if (!data) {
            return;
        }
        // Set fields length to data length
        this.fields.splice(data.length, this.fields.length - data.length);
        // Set fields values to data values
        data.forEach((dataField, i) => {
            if (this.fields[i]) {
                this.fields[i].setValue(dataField, {
                    interract,
                    isInitValue,
                });
            }
            else if (this.builder) {
                this.fields.push(this.builder(dataField, {
                    interract,
                    isInitValue,
                }));
            }
            else {
                throw Error("No builder defined");
            }
        });
        if (isInitValue) {
            this.initValue = this.value;
        }
    }
    get(index) {
        return this.fields[index];
    }
    get validation() {
        if (this.validate) {
            const validation = this.validate(this.value);
            if (!validation.valid) {
                return validation;
            }
        }
        const invalidField = this.fields.find((field) => !field.isValid);
        if (invalidField) {
            return invalidField.validation;
        }
        return {
            valid: true,
        };
    }
    get isValid() {
        return this.validation.valid;
    }
    get value() {
        return this.fields.map((field) => field.value);
    }
    get interacted() {
        return this.fields.some((field) => field.interacted);
    }
    get pristine() {
        // FIXME how to detect pristine state
        return fastDeepEqual(this.initValue, this.value);
    }
    get errorMessage() {
        return this.validation?.message;
    }
}
__decorate([
    mobx_1.observable,
    __metadata("design:type", Array)
], FieldsArray.prototype, "fields", void 0);
__decorate([
    mobx_1.observable,
    __metadata("design:type", Array)
], FieldsArray.prototype, "initValue", void 0);
__decorate([
    mobx_1.action("FIELD_ARRAY_RESET"),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], FieldsArray.prototype, "reset", null);
__decorate([
    mobx_1.action("FIELD_ARRAY_MAT"),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], FieldsArray.prototype, "markAsTouched", null);
__decorate([
    mobx_1.action("FIELD_ARRAY_MAUT"),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], FieldsArray.prototype, "markAsUntouched", null);
__decorate([
    mobx_1.action("FIELD_ARRAY_SETVALUE"),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array, Object]),
    __metadata("design:returntype", void 0)
], FieldsArray.prototype, "setValue", null);
__decorate([
    mobx_1.computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], FieldsArray.prototype, "validation", null);
__decorate([
    mobx_1.computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], FieldsArray.prototype, "isValid", null);
__decorate([
    mobx_1.computed,
    __metadata("design:type", Array),
    __metadata("design:paramtypes", [])
], FieldsArray.prototype, "value", null);
__decorate([
    mobx_1.computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], FieldsArray.prototype, "interacted", null);
__decorate([
    mobx_1.computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], FieldsArray.prototype, "pristine", null);
__decorate([
    mobx_1.computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], FieldsArray.prototype, "errorMessage", null);
exports.default = FieldsArray;
